/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-02 10:01:13.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.utils;

import org.apache.http.ssl.SSLContextBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author: QMTH
 * @since: 2018/11/2
 */
public class FileUtils {
    private static final Logger log = LoggerFactory.getLogger(FileUtils.class);

    public static String randomUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 生成日期目录
     * PS: yyyyMM
     */
    public static String dateDir() {
        final String pattern = "yyyyMM";
        return "/" + new SimpleDateFormat(pattern).format(new Date()) + "/";
    }

    /**
     * 获取文件后缀名(包含".")
     */
    public static String getFileSuffix(String filePath) {
        if (filePath == null) {
            return "";
        }
        int index = filePath.lastIndexOf(".");
        if (index > 0 && index < filePath.length() - 1) {
            return filePath.substring(index).toLowerCase();
        }
        return "";
    }

    /**
     * 生成新的文件名(带后缀)
     * PS: uuid.xyz
     */
    public static String newFileName(String fileName) {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        if (fileName != null) {
            int index = fileName.lastIndexOf(".");
            if (index > 0 && index < fileName.length() - 1) {
                //获取原来文件的后缀名
                return uuid + fileName.substring(index).toLowerCase();
            }
        }
        return uuid;
    }

    /**
     * 获取文件名(包含后缀名)
     */
    public static String getFileName(String filePath) {
        if (filePath == null) {
            return "";
        }
        filePath = filePath.replace("\\", "/");
        int index = filePath.lastIndexOf("/");
        if (index >= 0) {
            return filePath.substring(index + 1);
        }
        return filePath;
    }

    /**
     * 创建文件目录
     */
    public static boolean makeDirs(String path) {
        if (path == null || "".equals(path)) {
            return false;
        }
        File folder = new File(path);
        if (!folder.exists()) {
            return folder.mkdirs();
        }
        return true;
    }

    /**
     * 文件压缩
     *
     * @param target  待压缩的目录或文件
     * @param zipFile 压缩后的文件
     */
    public static boolean zip(File target, File zipFile) {
        if (target == null || !target.exists()) {
            log.error("目录或文件不能为空！");
            return false;
        }

        if (zipFile == null) {
            log.error("待压缩的文件不能为空！");
            return false;
        }

        // try-with-resources (since jdk1.7)
        try (OutputStream outStream = new FileOutputStream(zipFile);
             ZipOutputStream zipStream = new ZipOutputStream(outStream, Charset.forName("UTF-8"));) {
            //是否为文件夹
            if (target.isDirectory()) {
                File[] files = target.listFiles();
                if (files.length == 0) {
                    log.error("当前目录内未找到任何文件！");
                    return false;
                }

                for (File file : files) {
                    doZip(zipStream, file, null);
                }
            } else {
                doZip(zipStream, target, null);
            }
            return true;
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return false;
        }
    }

    private static void doZip(ZipOutputStream zipStream, File curFile, String dirName) throws IOException {
        if (dirName == null) dirName = "";
        String entryName = dirName + curFile.getName();
        //log.debug("entryName:" + entryName);

        if (curFile.isDirectory()) {
            String subDirName = entryName + File.separator;
            File[] files = curFile.listFiles();
            if (files.length > 0) {
                for (File file : files) {
                    doZip(zipStream, file, subDirName);
                }
            } else {
                //空文件夹暂不处理
                //zipStream.putNextEntry(new ZipEntry(subDirName));
                //zipStream.closeEntry();
            }
        } else {
            try (InputStream is = new FileInputStream(curFile);) {
                zipStream.putNextEntry(new ZipEntry(entryName));

                int len;
                byte[] bytes = new byte[1024];
                while ((len = is.read(bytes)) > 0) {
                    zipStream.write(bytes, 0, len);
                }

                zipStream.closeEntry();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    public static void saveURLToFile(String link, File file) {
        if (link == null || file == null) {
            log.warn("link or file must be not null.");
            return;
        }

        if (link.startsWith("https")) {
            saveHttpsURLToFile(link, file);
            return;
        }

        try {
            URL url = new URL(link);
            org.apache.commons.io.FileUtils.copyURLToFile(url, file);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private static void saveHttpsURLToFile(String link, File file) {
        SSLContext sslContext = null;
        try {
            //System.setProperty ("jsse.enableSNIExtension", "false");
            sslContext = new SSLContextBuilder().loadTrustMaterial(null, (chain, authType) -> true).build();
        } catch (NoSuchAlgorithmException e) {
            log.error(e.getMessage(), e);
        } catch (KeyStoreException e) {
            log.error(e.getMessage(), e);
        } catch (KeyManagementException e) {
            log.error(e.getMessage(), e);
        }

        if (sslContext == null) {
            log.warn("http sslContext must be not null.");
            return;
        }

        HttpsURLConnection connection;
        try {
            URL url = new URL(link);
            HttpsURLConnection.setDefaultSSLSocketFactory(sslContext.getSocketFactory());
            connection = (HttpsURLConnection) url.openConnection();
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(15000);

            if (HttpURLConnection.HTTP_OK != connection.getResponseCode()) {
                log.warn("http connection response fail." + connection.getResponseMessage());
                return;
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return;
        }

        //响应成功
        try (DataInputStream in = new DataInputStream(connection.getInputStream());
             DataOutputStream out = new DataOutputStream(new FileOutputStream(file));) {
            byte[] bytes = new byte[1024];
            int len;
            while ((len = in.read(bytes)) > 0) {
                out.write(bytes, 0, len);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }
    }

}