/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:30:57.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.ExamQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.ObjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.SubjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.enums.QuesStructType;
import cn.com.qmth.examcloud.core.print.repository.ObjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.repository.SubjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.service.ExamQuestionStructureService;
import cn.com.qmth.examcloud.core.print.service.ExamStructureService;
import cn.com.qmth.examcloud.core.print.service.StatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.examquestionstructure.ExamQuestionStructureInfo;
import cn.com.qmth.examcloud.core.print.service.bean.examstructure.ExamStructureInfo;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.*;

import static cn.com.qmth.examcloud.core.print.common.Constants.SYS_CODE_500;

/**
 * @author: QMTH
 * @since: 2018/10/17
 */
@Service
public class QuestionStructureServiceImpl implements ExamQuestionStructureService {
    private static final Logger log = LoggerFactory.getLogger(QuestionStructureServiceImpl.class);
    @Autowired
    private ObjectiveQuestionStructureRepository objectiveQuestionStructureRepository;
    @Autowired
    private SubjectiveQuestionStructureRepository subjectiveQuestionStructureRepository;
    @Autowired
    private ExamStructureService examStructureService;
    @Autowired
    private StatisticService statisticService;

    @Override
    public void savePaperQuestionStructure(@NotNull Long examId, @NotNull String paperId) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        //从题库端获取试卷的试题结构信息
        //PaperServiceImpl --> findQuestionStructure(paperId)
        ExamQuestionStructureInfo info = statisticService.findStructureByPaperId(examId, paperId);
        if (info == null) {
            log.warn("试卷试题结构不存在！ paperId is " + paperId);
            return;
        }

        //先清理旧数据
        objectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);
        subjectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);

        //保存客观题数据
        List<ObjectiveQuestionStructure> objectives = info.getObjectives();
        if (objectives != null && !objectives.isEmpty()) {
            for (ObjectiveQuestionStructure structure : objectives) {
                structure.setExamId(examId);
                structure.setPaperId(paperId);
            }
            objectiveQuestionStructureRepository.saveAll(info.getObjectives());
        }

        //保存主观题数据
        List<SubjectiveQuestionStructure> subjectives = info.getSubjectives();
        if (subjectives != null && !subjectives.isEmpty()) {
            for (SubjectiveQuestionStructure structure : subjectives) {
                structure.setExamId(examId);
                structure.setPaperId(paperId);
            }
            subjectiveQuestionStructureRepository.saveAll(info.getSubjectives());
        }
    }

    @Override
    public List<ObjectiveQuestionStructure> getObjectiveQuestionStructureList(Long orgId, Long examId, String paperId, String paperType) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<ObjectiveQuestionStructure> spec = SpecUtils.buildSearchers(ObjectiveQuestionStructure.class, searches.build());
        List<ObjectiveQuestionStructure> objectives = objectiveQuestionStructureRepository.findAll(spec);
        if (objectives == null || objectives.isEmpty()) {
            return Lists.newArrayList();
        }

        //获取考试结构设置的客观题数量
        ExamStructureInfo info = examStructureService.getExamStructure(orgId, examId);
        if (info == null || info.getQuestionStructure() == null) {
            throw new StatusException(SYS_CODE_500, "当前的考试结构设置不存在！");
        }

        //默认属性信息
        Map<String, String> properties = new HashMap<>();

        //按试题类型封装试题集合
        Map<String, List<ObjectiveQuestionStructure>> structureMaps = new HashMap<>();
        ExamQuestionStructure questionStructure = info.getQuestionStructure();
        for (ObjectiveQuestionStructure objective : objectives) {
            objective.setPaperType(paperType);//补全试卷类型
            List<ObjectiveQuestionStructure> list = structureMaps.get(objective.getQuestionType());
            if (list == null) {
                list = new ArrayList<>();
                structureMaps.put(objective.getQuestionType(), list);
            }
            list.add(objective);

            if (properties.size() > 0) {
                continue;
            }
            properties.put("courseCode", objective.getCourseCode());
            properties.put("courseName", objective.getCourseName());
            properties.put("paperName", objective.getPaperName());
            properties.put("paperType", paperType);
        }

        //补全阅卷时缺失的单选题
        int startNum = 1;
        this.fillQuestionStructure(structureMaps, properties, QuesStructType.SINGLE_ANSWER_QUESTION, questionStructure.getSingleChoiceTotal(), startNum);

        //补全阅卷时缺失的多选题
        startNum += questionStructure.getSingleChoiceTotal();
        this.fillQuestionStructure(structureMaps, properties, QuesStructType.MULTIPLE_ANSWER_QUESTION, questionStructure.getMultipleChoiceTotal(), startNum);

        //补全阅卷时缺失的判断题
        startNum += questionStructure.getMultipleChoiceTotal();
        this.fillQuestionStructure(structureMaps, properties, QuesStructType.BOOL_ANSWER_QUESTION, questionStructure.getBoolQuestionTotal(), startNum);

        //返回所有客观题集合
        List<ObjectiveQuestionStructure> allObjectives = new ArrayList<>();
        for (String questionType : structureMaps.keySet()) {
            allObjectives.addAll(structureMaps.get(questionType));
        }

        //按大题号和小题号排序
        Collections.sort(allObjectives, (current, target) -> {
            if (current.getUnitNum() > target.getUnitNum()) {
                return 1;
            } else if (current.getUnitNum() < target.getUnitNum()) {
                return -1;
            }
            return 0;
        });

        return allObjectives;
    }

    /**
     * 补全客观题的阅卷纪录
     *
     * @See ExportPaperAbstractService --> fillObjectiveQuestions(...)
     */
    private void fillQuestionStructure(Map<String, List<ObjectiveQuestionStructure>> structureMaps, Map<String, String> properties, QuesStructType quesStructType, int limitSize, int startNum) {
        List<ObjectiveQuestionStructure> objectives = structureMaps.get(quesStructType.getTitle());
        if (objectives == null) {
            objectives = new ArrayList<>();
        }

        final int currentSize = objectives.size();

        if (limitSize == 0) {
            //若当前题型尚未定义试题数量，则置空当前小题集合
            objectives.clear();
            return;
        }

        if (currentSize == 0) {
            //若当前试卷该题型的数量为0，则设置默认小题集合
            for (int i = 0; i < limitSize; i++) {
                ObjectiveQuestionStructure objective = new ObjectiveQuestionStructure(quesStructType, properties);
                objective.setUnitNum(startNum + i);
                objectives.add(objective);
            }
            return;
        }

        if (limitSize > currentSize) {
            //若当前题型定义的试题数量“大于”当前试卷该题型的数量，则修正小题序号并补全剩余的小题
            for (int i = 0; i < limitSize; i++) {
                if (i < currentSize) {
                    ObjectiveQuestionStructure objective = objectives.get(i);
                    objective.setUnitNum(startNum + i);
                } else {
                    ObjectiveQuestionStructure objective = new ObjectiveQuestionStructure(quesStructType, properties);
                    objective.setUnitNum(startNum + i);
                    objectives.add(objective);
                }
            }
            return;
        }

        if (limitSize < currentSize) {
            //若当前题型定义的试题数量“小于”当前试卷该题型的数量，则修正小题序号并移除多余的小题
            List<ObjectiveQuestionStructure> list = new ArrayList<>();
            for (int i = 0; i < limitSize; i++) {
                ObjectiveQuestionStructure objective = objectives.get(i);
                objective.setUnitNum(startNum + i);
                list.add(objective);
            }
            structureMaps.put(quesStructType.getTitle(), list);
            return;
        }

        //当前题型定义的试题数量“等于”当前试卷该题型的数量，则仅修正小题序号即可
        for (int i = 0; i < limitSize; i++) {
            ObjectiveQuestionStructure objective = objectives.get(i);
            objective.setUnitNum(startNum + i);
        }
    }

    @Override
    public List<SubjectiveQuestionStructure> getSubjectiveQuestionStructureList(Long orgId, Long examId, String paperId, String paperType) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<SubjectiveQuestionStructure> spec = SpecUtils.buildSearchers(SubjectiveQuestionStructure.class, searches.build());
        List<SubjectiveQuestionStructure> subjectives = subjectiveQuestionStructureRepository.findAll(spec);
        if (subjectives == null || subjectives.isEmpty()) {
            return Lists.newArrayList();
        }

        for (SubjectiveQuestionStructure structure : subjectives) {
            structure.setPaperType(paperType);//补全试卷类型
        }
        return subjectives;
    }

    @Override
    public void syncObjectiveCourseNameByCourseId(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        objectiveQuestionStructureRepository.updateCourseNameByCourseId(courseId, courseName);
    }

    @Override
    public void syncSubjectiveCourseNameByCourseId(Long courseId, String courseName) {
        if (courseId == null || StringUtils.isBlank(courseName)) {
            return;
        }
        subjectiveQuestionStructureRepository.updateCourseNameByCourseId(courseId, courseName);
    }

}